from setuptools import setup

setup(
    name="ShakenCBLattice",
    version="0.1.0",
    description="Package used to perform band structure caluclations on periodically driven Checkerboard optical lattice",
    author="James Maslek",
    author_email="jmaslek11@gmail.com",
    packages=["ShakenCBLattice"],
)
